module w06_cdc(
  input CLOCK_50,     // clock 50 MHz   (PIN_AF14)
  input CLOCK2_50,    // clock_2 50 MHz (PIN_AA16)
  input [3:0] KEY,
  input [9:0] SW,
  output [35:0] dout  // safe dummy output
);

  parameter IS_SIMULATION = 0; // 1 for simulation, 0 for real hardware

  wire clk_src = CLOCK_50;    // connect clk directly to external 50 MHz clock

  //---------------------------------------------------------------------------
  // signal source domain
  //---------------------------------------------------------------------------
  wire signed [11:0] sig_raw;
  wire signed [15:0] sig_src = {sig_raw, 4'd0};

  //
  // sinewave signal source - frequency 100 kHz
  //
  
  // set NCO frequency : Phase_inc = (Fout/Fclk)*2^32
  localparam [31:0] C_PHASE_INC = 32'd8589934;   // 100 kHz
  wire [31:0] phase_inc = C_PHASE_INC;

  waveform_gen waveform_gen_inst(
    .clk(clk_src),
    .reset(1'b1),     // reset is active low
    .en(1'b1),
    .phase_inc(phase_inc),
    // output signals
    .sin_out(sig_raw),
    .cos_out(),
    .squ_out(),
    .saw_out()
  );

  //
  // clock enable signal
  //
  wire ce_src;
  ce_gen #(
    .DIV_FACT(16)
  ) ce_gen_instance(
    .clk(clk_src),
    .rst(1'b0),
    .ce(1'b1),
    .ce_o(ce_src)
  );

  //---------------------------------------------------------------------------
  // signal destination domain
  //---------------------------------------------------------------------------
  (* noprune *) (* keep *) wire clk_dst;

  // destination clock source
  clk_gen clk_gen_instance(
    .refclk(clk_src),
    .rst(1'b0),
    .outclk_0(clk_dst)
  );

  //
  // sinewave signal
  //

  // two flops to remove metastability
  reg signed [15:0] sig_i;
  reg signed [15:0] sig_dst;
  
  always @(posedge clk_dst) begin
    sig_i <= sig_src;
    sig_dst <= sig_i;
  end

  //
  // clock enable
  //
  reg ce_i;
  reg ce_dst;
  reg ce_dst2;

  always @(posedge clk_dst) begin
    ce_i <= ce_src;
    ce_dst <= ce_i;
    ce_dst2 <= ce_dst;
  end


  // safe dummy output
  assign dout = {ce_dst2, ce_dst, sig_dst};

endmodule
