`timescale 1ns / 1ns

module w06_cdc_tb();
  localparam CLK_PER = 10;
  
  // generate clock
  reg clk;
  initial begin
    clk = 1'b0;
    forever #(CLK_PER/2) clk = ~clk;
  end

  // instantiate the design under test (DUT)
  wire [3:0] KEY = 4'b1111;
  wire AUD_ADCDAT = 1'b0;
  wire AUD_BCLK = 1'b0;
  wire AUD_ADCLRCK = 1'b0;
  wire AUD_DACLRCK = 1'b0;
  wire FPGA_I2C_SDAT = 1'bz;
  wire FPGA_I2C_SCLK = 1'bz;
  wire AUD_XCK = 1'b0;
  wire AUD_DACDAT = 1'b0;
  reg [9:0] SW;

  w06_cdc #(
    .IS_SIMULATION(1)
  ) dut(
    .CLOCK_50(clk),
    .CLOCK2_50(clk),
    .KEY(KEY),
    .SW(SW),
    .AUD_ADCDAT(AUD_ADCDAT),
    .AUD_BCLK(AUD_BCLK),
    .AUD_ADCLRCK(AUD_ADCLRCK),
    .AUD_DACLRCK(AUD_DACLRCK),
    .FPGA_I2C_SDAT(FPGA_I2C_SDAT),
    .FPGA_I2C_SCLK(FPGA_I2C_SCLK),
    .AUD_XCK(AUD_XCK),
    .AUD_DACDAT(AUD_DACDAT)
  );


  reg sim_finished = 1'b0;

  //---------------------------------------------------------------------------
  // generate stimulus and save filter input and output to files
  //---------------------------------------------------------------------------
  localparam AUDIO_DATA_WIDTH = dut.audio_module_inst.AUDIO_DATA_WIDTH;
  wire [AUDIO_DATA_WIDTH-1:0] filtin_data = dut.audio_module_inst.filtin_data;
  wire [AUDIO_DATA_WIDTH-1:0] filtout_data = dut.audio_module_inst.filtout_data;
  wire src_dv = dut.audio_module_inst.filtin_dv;
  wire filtout_dv = dut.audio_module_inst.filtout_dv;


  initial begin
    SW = 10'b00_0000_0101;

    // wait some time until the reset is released
    @(negedge clk);
    #(CLK_PER*1000);

    SW = 10'b00_0000_0111;
    
    // TODO: save filter input and output here
    #(CLK_PER*10000);

    sim_finished = 1'b1;
    #(CLK_PER*10);
    $finish;
  end

  // save filter input to a file
  integer fid_filtin;
  initial begin
    // open the file
    fid_filtin = $fopen("../../sources/data/filter_din.txt", "w");
    if (fid_filtin == 0) begin
        $display("ERROR: File could not be opened.");
        $finish;
    end

    while (!sim_finished) begin
        // Check if the data is valid
        if (src_dv) begin
            // Write the data to file
            $fdisplay(fid_filtin, "%d", filtin_data);
        end
        #(CLK_PER);         // wait for one clock period
    end

    $fclose(fid_filtin);      // close the file
  end

  // save filter output to a file
  integer fid_filtout;
  initial begin
    // open the file
    fid_filtout = $fopen("../../sources/data/filter_dout.txt", "w");
    if (fid_filtout == 0) begin
        $display("ERROR: File could not be opened.");
        $finish;
    end

    while (!sim_finished) begin
        // Check if the data is valid
        if (filtout_dv) begin
            // Write the data to file
            $fdisplay(fid_filtout, "%d", filtout_data);
        end
        #(CLK_PER);         // wait for one clock period
    end

    $fclose(fid_filtout);      // close the file
  end


endmodule
